// file: PointImpl.java
// author: Bob Muller
// date: April 30, 2013
//
// This file contains both an interface (an API) and an implementation of
// a simple ADT for points in a Cartesian plane.
//
// The API.
//
interface Point {  
    public double getX();
    public double getY();
    public String toString();
    public double compareTo(Point other);
}

// An -implementation- of the Point API.
//
public class PointImpl implements Point {  // NB: the 'implements Point'
                                           // tells the java compiler to 
                                           // confirm that PointImpl actually
                                           // has everything specified in the
                                           // Point interface.
 
    private double x;           // NB: These two instance variables are hidden!!!
    private double y;           // The only way to use a Point is through the
                                // functions/methods specified in the API.
    
    // A constructor for PointImpl. (Corresponds to __init__ in Python.)
    //
    public PointImpl(double x, double y) {
        this.x = x;
        this.y = y;
    }
    
    // Getters.
    //
    public double getX() { return this.x; }
    public double getY() { return this.y; }
    
    public String toString() { return "(" + x + ", " + y + ")"; }
    
    // Lexicographic comparison of Points.
    //
    public double compareTo(Point other) {    // NB: The type of other!
        
        double xAnswer = this.getX() - other.getX();
        if (xAnswer != 0.0)
            return xAnswer;
        else
            return this.getY() - other.getY();
    }
}