(* file: point.ml
 * author: Bob Muller
 * date: November 30, 2014
 * 
 * This file contains a specification (signature) and the
 * implementation (structure) of a simple ADT for points in
 * the 2D plane. The specification is usually contained in
 * a separate .mli file. See the implementation in ../file/
*)
    type t = {x:float; y:float};;

    let make x y = {x=x; y=y};;

    let getX p = p.x;;
    let getY p = p.y;;

    let toString p = 
      let xs = string_of_float (getX p) in
      let ys = string_of_float (getY p) 
      in
	"(" ^ xs ^ ", " ^ ys ^ ")";;

    let distance p = sqrt((getX p) ** 2.0 +. (getY p) ** 2.0);;

    (* sign : float -> {-1, 0, +1}
     *
     * (sign x) determines the sign of x. An internal helper.
     *)
    let sign float = 
      match (float < 0.0, float = 0.0) with
	  (true, _) -> -1
	| (false, true) -> 0
	| (false, false) -> 1;;

    (* A lexicographic ordering *)

    let compareTo p1 p2 =
      let (x1, y1) = (getX p1, getY p1) in
      let (x2, y2) = (getX p2, getY p2) in
      let xSign = sign (x1 -. x2)
      in
	match xSign != 0 with
	    true  -> xSign
	  | false -> sign (y1 -. y2);;
    
    (* Forcing equal to be consistent with compareTo by fiat. *)

    let equal p1 p2 = (compareTo p1 p2) = 0;;
    
    (* These points are immutable. *)

    let move point dx dy =
      make ((getX point) +. dx) ((getY point) +. dy);;
      


    
